-- =====================================================================
-- Copyright  2010-2011 by Cryptographic Engineering Research Group (CERG),
-- ECE Department, George Mason University
-- Fairfax, VA, U.S.A.
-- =====================================================================

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use work.sha3_pkg.all;

entity bshift is   
	generic ( 
		w 			: integer := 16;	
		left	 	: integer := 1 
	);
	port (						  
		ii   : in  std_logic_vector(w-1 downto 0);
		rtr : in  std_logic_vector(log2(w)-1 downto 0);
		oo   : out std_logic_vector(w-1 downto 0) );

end bshift;

--}} End of automatically maintained section

architecture struct of bshift is
	constant lw : integer := log2(w);
	type temp_i_type is array (0 to lw) of std_logic_vector(w-1 downto 0);
	signal itemp : temp_i_type;
	
begin								  
	itemp(0) <= ii;

	LEFT_SHIFT_GEN:
	if left = 1 generate
		barrel_gen : for k in 1 to lw generate
			itemp(k) <= itemp(k-1) when (rtr(k-1) = '0') else ( itemp(k-1)(w-(2**(k-1))-1 downto 0) & itemp(k-1)(w-1 downto w-(2**(k-1))));
		end generate;
	end generate;
	
	RIGHT_SHIFT_GEN:
	if left = 0 generate
		barrel_gen : for k in 1 to lw generate
			itemp(k) <= itemp(k-1) when (rtr(k-1) = '0') else ( itemp(k-1)( (2**k)-1 downto 0 )	& itemp(k-1)(w-1 downto 2**k));
		end generate;
	end generate;

	oo <= itemp(lw);

end struct;
